# -*- coding: utf-8 -*-


import xbmc,xbmcgui
import json,os
import threading
from resources.lib.modules import control
from resources.lib.modules.log_utils import log

COOKIE = os.path.join(control.dataPath,'TVMcookies')


class Monitor(xbmc.Monitor):
    def __init__( self, *args, **kwargs ):
        xbmc.Monitor.__init__( self )

        

    def onSettingsChanged( self ):
        player.reload_settings()
        


class Player(xbmc.Player):
    
    def reload_settings(self):
        perc = control.setting('perc')
        if perc=='':
            control.openSettings('Account')
            return
        self._min_percent = int(perc)
        from resources.lib.modules import tvmaze
        maze=tvmaze.MazeLogin()
        self.auto_rate = control.setting('auto_rate')=='true'

    def _setUp(self):
        self._playbackLock.set()
        self._tracker = threading.Thread(target=self._trackPosition)

    def __init__(self):
        
        xbmc.Player.__init__( self )
        log('Service: starting...')
        self._totalTime = 999999
        self._lastPos = 0
        self._tracker = None
        self._playbackLock = threading.Event()
        self._monitor = Monitor()
        self.reload_settings()
        
    def onPlayBackStopped(self):
        try:
            xbmcgui.Window(10000).clearProperty('script.trakt.ids')
            log('Minimum percent: %s'%self._min_percent)
            actual_percent = (self._lastPos/self._totalTime)*100
            log('lastPos / totalTime : %s / %s = %s %%' % (self._lastPos,
                self._totalTime, actual_percent))
            if actual_percent<self._min_percent:
                return
            from resources.lib.modules import tvmaze
            maze=tvmaze.MazeLogin()
            marked = maze.mark_watched(self.data)
            if marked:
                if self.auto_rate:
                    maze.rate_item(self.data)
        except:
            pass


    def onPlayBackEnded(self):
        xbmcgui.Window(10000).clearProperty('script.trakt.ids')
        self.onPlayBackStopped()

    def onPlayBackStarted(self):
        xbmc.sleep(1000)
        self._setUp()
        self._tracker.start()
        self._totalTime = self.getTotalTime()
        self.data = {}
        if self.isPlayingVideo():
            # get item self.data from json rpc
            result = control.kodiJsonRequest({'jsonrpc': '2.0', 'method': 'Player.GetItem', 'params': {'playerid': 1}, 'id': 1})
            
            try:
                _filename = self.getPlayingFile()
                if _filename.startswith('pvr://') and control.setting('exclude_live')=='true':
                    log('Live TV playing - returning')
                    return
            except:
                pass

            type = result['item']['type']

            if 'id' not in result['item']:
                # do a deeper check to see if we have enough self.data to perform scrobbles
                log("[TV Maze] - Started playing a non-library file, checking available self.data.")

                season = xbmc.getInfoLabel('VideoPlayer.Season')
                episode = xbmc.getInfoLabel('VideoPlayer.Episode')
                showtitle = xbmc.getInfoLabel('VideoPlayer.TVShowTitle')
                year = xbmc.getInfoLabel('VideoPlayer.Year')
                #get trakt ids if possible
                video_ids = xbmcgui.Window(10000).getProperty('script.trakt.ids')
                if video_ids:
                    self.data['video_ids'] = json.loads(video_ids)
                log("info - ids: %s, showtitle: %s, Year: %s, Season: %s, Episode: %s" % (video_ids, showtitle, year, season, episode))

                if season and episode and (showtitle or video_ids):
                    # we have season, episode and either a show title or video_ids, can scrobble this as an episode
                    self.data['season'] = int(season)
                    self.data['episode'] = int(episode)
                    self.data['showtitle'] = showtitle
                    self.data['title'] = xbmc.getInfoLabel('VideoPlayer.Title')
                    if year.isdigit():
                        self.data['year'] = int(year)
                    log("Playing a non-library 'episode' - %s - S%02dE%02d - %s." % (self.data['showtitle'], self.data['season'], self.data['episode'], self.data['title']))
                elif (year or video_ids) and not season and not showtitle:
                    #movie
                    return
                
                elif showtitle:
                    title, season, episode = control.regex_tvshow(showtitle)
                    self.data['season'] = season
                    self.data['episode'] = episode
                    self.data['title'] = self.data['showtitle'] = title
                    log("Title: %s, showtitle: %s, season: %d, episode: %d" % (title, showtitle, season, episode))
                else:
                    log("Non-library file, not enough self.data for scrobbling, skipping.")
                    return

            elif type=='episode':
                id = result['item']['id']
                result = control.kodiJsonRequest({'jsonrpc': '2.0', 'method': 'VideoLibrary.GetEpisodeDetails', 'params': {'episodeid': id, 'properties': ['tvshowid', 'season', 'episode', 'file']}, 'id': 1})
                if result:
                    tvshowid = int(result['episodedetails']['tvshowid'])
                    season = int(result['episodedetails']['season'])
                    episode = int(result['episodedetails']['episode'])
                    currentfile = result['episodedetails']['file']

                    result = control.kodiJsonRequest({'jsonrpc': '2.0', 'method': 'VideoLibrary.GetTVShowDetails', 'params': {'tvshowid': tvshowid, 'properties':['year', 'imdbnumber']}, 'id': 1})
                    if result:
                        showtitle = result['tvshowdetails']['label']
                        try:
                            tvdb = result['tvshowdetails']['imdbnumber']
                            self.data['video_ids'] = {'tvdb': tvdb}
                        except:
                            pass
                        year = result['tvshowdetails']['year']
                        
                        self.data['season'] = season
                        self.data['episode'] = episode
                        self.data['title'] = self.data['showtitle'] = showtitle
                        self.data['year'] = year

                        
            


        
    def _trackPosition(self):
        while self._playbackLock.isSet() and not xbmc.abortRequested:
            try:
                self._lastPos = self.getTime()
            except:
                 self._playbackLock.clear()

            xbmc.sleep(250)
        
    
        

    def _setUp(self):
        self._playbackLock.set()
        self._tracker = threading.Thread(target=self._trackPosition)
 

if ( __name__ == "__main__" ):
    player = Player()



    while not xbmc.abortRequested:
          xbmc.sleep(100)