import requests
from BeautifulSoup import BeautifulSoup as bs
import re
import urllib
import json
import xbmc,pickle
import cache,client,control,os
from log_utils import log
import pytvmaze
COOKIE = os.path.join(control.dataPath,'TVMcookies')
HEADERS = {'User-agent':client.agent()}
import xbmc,xbmcgui
#login class & user specific methods
class MazeLogin():

	def load_cookies(self):
		try:
			with open(COOKIE,'rb') as f:
				cookies = pickle.load(f)
				return cookies
		except:
			return False

	def login(self):
		#get session token
		log('Getting session token...')
		

		session = requests.Session()
		session.headers = HEADERS
		html = session.get(self.base_url).text
		try:
			self.token = bs(html).find('input',{'type':'hidden'})['value']
		except:
			self.token = re.findall('csrf-token[\"\'] content=[\"\']([^\"\']+)[\"\']',html)[0]
		
		control.set_setting('tvm_token',self.token)
		#login to tvmaze
		payload = {'_csrf': self.token,'LoginForm[password]':self.password,'LoginForm[username]':self.username,'LoginForm[rememberMe]':'0', 'LoginForm[rememberMe]':'1'}
		check = session.post(self.base_url, data=payload)
		check = ((check.text).encode('ascii','ignore'))
		if 'Unknown username/email' in check:
			log('Unknown username/email')
			login_error=True
			login_message='Unknown username/email'
			control.infoDialog(login_message)
		elif 'Incorrect password' in check:
			log('Incorrect password')
			login_error=True
			login_message='Incorrect password'
			control.infoDialog(login_message)
		else:
			login_error=False
			login_message='Logged in successfully'
			control.infoDialog(login_message)

		
			with open(COOKIE, 'wb') as f:
				pickle.dump(session.cookies, f)


	def get_session(self):
		session = requests.Session()
		session.headers = HEADERS
		cookies = self.load_cookies()
		session.headers.update({'X-CSRF-Token':control.setting('tvm_token')})

		if not cookies:
			self.login()
			session.cookies = self.load_cookies()
			session.headers.update({'X-CSRF-Token':control.setting('tvm_token')})
			return session

		session.cookies = cookies
		not_logged = 'Login | Keep track of your favorite shows' in session.get('http://www.tvmaze.com/dashboard').text
		if not_logged:
			self.login()
			session.cookies = self.load_cookies()
			session.headers.update({'X-CSRF-Token':control.setting('tvm_token')})
			return session
				
		log('Using old session cookies')
		return session



	#initial login, create login session
	#args: username, password -> tvmaze login information
	def __init__(self,):
		self.auto_follow = control.setting('auto_follow')
		self.username = control.setting('username')
		self.password = control.setting('password')
		
		
		self.headers = {}
		self.base_url = 'http://www.tvmaze.com/site/login'
		self.token = control.setting('tvm_token')
		self.headers = HEADERS
		if self.token!='':
			self.headers.update({'X-CSRF-Token':self.token})

		self.session = self.get_session()
		
		



	#follow/unfollow 
	#args: id -> tvmaze id of the item (either show,person,network or web channel)
	#kwargs: type (int/str 1,2,3 or 4 ) -> 1 for shows, 2 for people, 3 for networks, 4 for web_channels (default is 1)
	#returns: 0 if failed, 1 if item followed, -1 if item unfollowed
	def toggle_follow(self,id,type='1'):
		self.session = self.get_session()
		types=['http://www.tvmaze.com/follow/toggle?show_id=%s&widget=small','http://www.tvmaze.com/follow/toggle?person_id=%s&widget=small','http://www.tvmaze.com/follow/toggle?network_id=%s&widget=small','http://www.tvmaze.com/follow/toggle?webChannel_id=%s&widget=small']

		url=types[int(type)-1] % id
		#show_url='http://www.tvmaze.com/follow/toggle?show_id=%s&widget=small' %tvshow_id

		#post the follow
		follow = self.session.post(url,headers=self.headers).text
		if 'page not found' in follow.lower() or 'error occured' in follow.lower():
			log('Wrong post data or some other error. (404 Page Not Found)')
			return 0
		else:
			method=bs(follow).find('a')['title']

			if method == 'Click to follow':
				log ('Unfollowed (id= %s)'%id)
				return -1
			elif method =='Click to unfollow':
				log('Followed (id= %s)'%id)
				self.followed_num+=1
				return 1



	def rate_item(self,data):

		season = data['season']
		episode = data['episode']
		showtitle = data['showtitle']
		try:
			year = data['year']
		except:
			year = None

		try:
			ids = data['video_ids']
		except:
			ids = None

		show = None
		if ids:
			try:
				imdb = ids['imdb']
			except:
				imdb = None

			try:
				tvdb = ids['tvdb']
			except:
				tvdb = None

			if imdb:
				show = pytvmaze.get_show(imdb_id=imdb)
			elif tvdb:
				show = pytvmaze.get_show(tvdb_id=tvdb)

		if not show:
			if year:
				show = pytvmaze.get_show(show_name=showtitle, show_year=year, embed='episodes')
			else:
				show = pytvmaze.get_show(show_name=showtitle,embed='episodes')


		episode = pytvmaze.episode_by_number(show.maze_id,season,episode)
		
		ep_id = episode.maze_id

		headers = {'Referer':episode.url}
		self.session.headers.update(headers)
		gui = RatingDialog(
			"script-tvmaze-RatingDialog.xml",
			control.addonPath,
			show=show.name,
			season=episode.season_number,
			episode=episode.episode_number,
			title=episode.title
			)

		gui.doModal()
		if gui.rating:
			self.session = self.get_session()
			post = 'http://www.tvmaze.com/vote/vote?episode_id=%s'%(ep_id)
			data = {'vote' : gui.rating}
			self.session.post(post, data=data)
			control.infoDialog('Rating submitted!')


	#mark watched/unwatched
	#args : episode_id 
	#returns 1 if marked as watched, -1 if marked as unwatched
	def toggle_episode_watch(self,episode_id, type='0'):
		ep = pytvmaze.episode_by_id(episode_id)
		ep_url = ep.url
		self.session = self.get_session()
		self.session.headers.update({'Referer':ep_url})
		ep_url='http://www.tvmaze.com/watch/set?episode_id=%s'%episode_id

		data = {'type': type}
		watched=self.session.post(ep_url,data=data).text

		if type=='0':
			log ('Marked episode as watched (id= %s)'%episode_id)
			return 1
		else :
			log('Marked episode as unwatched (id= %s)'%episode_id)
			return -1

		

	#check if user follows an item
	#args: id -> tvmaze id of the item
	#kwargs: type (int/str 1,2,3 or 4 ) -> 1 for shows, 2 for people, 3 for networks, 4 for web_channels (default is 1)
	#returns True if followed, False if not followed
	def isFollowed(self,id,type='1'):
		types = {'1':'shows','2':'people','3':'networks','4':'web_channels'}
		items = self.getFollowed(types[type])
		return str(id) in items

	#check if user watched an episode
	#args: episode_id ->
	#returns True if watched, False if not watched
	def isWatched(self,episode_id):
		ep = pytvmaze.episode_by_id(episode_id)
		ep_url = ep.url
		self.session = self.get_session()

		content = self.session.get(ep_url).text
		method=bs(content).find('div',{'id':'watch-tristate'}).findNext('span',{'class':'hide'})
		if 'unwatched' in method.getText().lower():
			log('isWatched (%s) = True'%episode_id)
			return True
		else :
			log('isWatched (%s) = False'%episode_id)
			return False

	#get url for ical .ics file (users personal calendar)
	#returns string url to the ics file, False if failed
	def get_calendar(self):
		self.session = self.get_session()
		url='http://www.tvmaze.com/dashboard/index'
		html=self.session.get(url).text
		try:
			ical_url=bs(html).find('div',{'id':'feed-panel'}).findAll('p')[1].find('a')['href']
			log('Calendar url:'+ ical_url)
			return ical_url
		except:
			return False


	#get list of followed items
	#args: type ->str 'shows', 'people', 'networks' or 'web_channels'
	#kwargs: sort -> sort method: 1 for 'Followed least recently', 2 for 'Followed most recently', 3 for 'A to Z' and 4 for 'Z to A'
	#returns array of tvmaze ids
	def getFollowed(self,type,sort='1'):
		self.session = self.get_session()
		url='http://www.tvmaze.com/dashboard/follows?sort=%s'%sort
		html=self.session.get(url).text
		tags=bs(html).find('div',{'id':'follow-list'})
		if type=='shows':
			reg='show_id=(\d+)'
		elif type=='people':
			reg='/people/(\d+)/'
		elif type=='networks':
			reg='/networks/(\d+)/'
		elif type=='web_channels':
			reg='/webchannels/(\d+)/'

		items=re.compile(reg).findall(str(tags))
		return items
		

	#get list of unwatched episodes for show
	#args: tvshow_id -> tvmaze id for a wanted show
	#returns an array of tvmaze episode ids
	def get_unwatched_episodes(self,tvshow_id):
		self.session = self.get_session()
		url='http://www.tvmaze.com/watch/show?show_id=%s'%tvshow_id
		html=self.session.get(url).text
		items=bs(html).find('div',{'class':'watch-list'}).find('tbody').findAll('tr')
		for item in items:
			watchstate=item.find('i')['class']
			if 'active' not in watchstate:
				id=item['data-key']
				ids+=[id]
		return ids


	def mark_watched(self,data):
			self.session = self.get_session()
			
			season = data['season']
			episode = data['episode']
			showtitle = data['showtitle']
			try:
				year = data['year']
			except:
				year = None

			try:
				ids = data['video_ids']
			except:
				ids = None

			show = None
			if ids:
				try:
					imdb = ids['imdb']
				except:
					imdb = None

				try:
					tvdb = ids['tvdb']
				except:
					tvdb = None

				if imdb:
					show = pytvmaze.get_show(imdb_id=imdb)
				elif tvdb:
					show = pytvmaze.get_show(tvdb_id=tvdb)

			if not show:
				if year:
					show = pytvmaze.get_show(show_name=showtitle, show_year=year, embed='episodes')
				else:
					show = pytvmaze.get_show(show_name=showtitle,embed='episodes')


			episode = pytvmaze.episode_by_number(show.maze_id,season,episode)
			show_id = show.maze_id
			ep_id = episode.maze_id
			if self.auto_follow:
				if not self.isFollowed(show_id):
					self.toggle_follow(show_id)
					control.infoDialog('Now following %s'%show.name)

			if not self.isWatched(ep_id):
				log('Episode %s is not watched, setting it to watched.'%ep_id)
				a = self.toggle_episode_watch(ep_id)
				control.infoDialog('%s S%sE%s marked as watched on your TV Maze account!'%(show,episode.season_number,episode.episode_number))
				return True
			else:
				log('Episode %s already watched, pass...'%ep_id)
				control.infoDialog('Episode already marked as watched.')	
				return True
		

	


class RatingDialog(xbmcgui.WindowXMLDialog):
    buttons = {
        11030: 1,
        11031: 2,
        11032: 3,
        11033: 4,
        11034: 5,
        11035: 6,
        11036: 7,
        11037: 8,
        11038: 9,
        11039: 10
    }

    focus_labels = {
        11030: "Weak Sauce :(",
        11031: "Terrible",
        11032: "Bad",
        11033: "Poor",
        11034: "Meh",
        11035: "Fair",
        11036: "Good",
        11037: "Great",
        11038: "Superb",
        11039: "Totally Ninja!"
    }

    def __init__(self, xmlFile,resourcePath,show=None,season=None,episode=None,title=None):
        self.rating = None
        self.default_rating = 5
        self.show = show
        self.season = season
        self.episode = episode
        self.title = title

    def onInit(self):
        s = "%s S%0dE%0d: %s" %(self.show,self.season,self.episode,self.title)
        self.getControl(10012).setLabel(s)

        rateID = 11029 + self.default_rating
        
        self.setFocus(self.getControl(rateID))

    def onClick(self, controlID):
        if controlID in self.buttons:
            self.rating = self.buttons[controlID]
            self.close()

    def onFocus(self, controlID):
        if controlID in self.focus_labels:
            s = self.focus_labels[controlID]

            

            self.getControl(10013).setLabel(s)
        else:
            self.getControl(10013).setLabel('')